function [ThetaA,NLLA,ImplicitStatesA] = AnalyticalPowell( ForecastsA,FTol,Gamma0,Gamma,Returns,SigmaSA,SA,ThetaA,Xinf,XTol,Yields )
%Powell's direction set method for finding a minimum NLL without derivatives
%See Numerical Recipes for algorithm
% Theta is the starting parameter vector on input and the optimal vector on ouput
% xi is the starting direction set on input and the then current direction set on ouput
% Tol is the convergence tolerance
% MaxIter is the maximum number of iterations
    Ndim = length(ThetaA);
    MaxIter = 2*Ndim;
    Iter = 1;
    Xi = eye(Ndim);
    NLLA= AnalyticalNLL(ThetaA,ForecastsA,Gamma0,Gamma,Returns,SigmaSA,SA,Xinf,Yields);
    LogLikelihoodA = -NLLA;
    display(LogLikelihoodA)
    while Iter <= MaxIter
        ThetaAt = ThetaA;
        NLLt = NLLA;
        for ii = 1:Ndim;%iterate over row vectors
            [ThetaA]=AnalyticalBrent(ForecastsA,Gamma0,Gamma,Returns,SigmaSA,SA,ThetaA,Xi(ii,:),Xinf,XTol,Yields);
        end
        [NLLA,ImplicitStatesA]= AnalyticalNLL(ThetaA,ForecastsA,Gamma0,Gamma,Returns,SigmaSA,SA,Xinf,Yields);
        LogLikelihoodA= -NLLA;
        display(LogLikelihoodA)
        if (abs(NLLt-NLLA) <= FTol );
            [NLLA,ImplicitStatesA]= AnalyticalNLL(ThetaA,ForecastsA,Gamma0,Gamma,Returns,SigmaSA,SA,Xinf,Yields);
            return
        end
        for ii = 1:Ndim-1;
            Xi(ii,:) = Xi(ii+1,:);
        end
        xi = ThetaA - ThetaAt;
        Xi(Ndim,:) = xi/norm(xi);%new normalized direction vector    xit = xit/norm(xit);%new normalized direction vector
        [ThetaA,NLLA]=AnalyticalBrent(ForecastsA,Gamma0,Gamma,Returns,SigmaSA,SA,ThetaA,xi,Xinf,XTol,Yields);
        Index = floor(Iter/Ndim);
        if( Iter/Ndim == Index);%refresh Xi after every Ndim iterations
            %tempsave = strcat('AnalyticalPowell',num2str(Index));
            %save(tempsave,'Xi');
            Xi = eye(Ndim);
        end
        Iter = Iter +1 ;
        save AnalyticalPowellFed Iter Xi ThetaA
    end
end        
function [ThetaA,NLLA] = AnalyticalBrent (ForecastsSP,Gamma0,Gamma,Returns,SigmaSA,SSP,ThetaA,xi,Xinf,XTol,Yields)

%*****************************************************************************80
%
% Brent seeks a local minimum of a function F(X) in an interval [A,B].
%
%
%  C is the square of the inverse of the golden ratio.
%
GOLD = 0.5 * ( 3.0 - sqrt ( 5.0 ) );
MaxIter = 100;
tol1 = XTol;
t2 = 2.0 * tol1;

%Bracket inital minimum
[ax,x,cx,~,NLLA] = AnalyticalBracket(ForecastsSP,Gamma0,Gamma,Returns,SigmaSA,SSP,ThetaA,xi,Xinf,Yields );

if(ax<cx)
    a = ax;
else
    a = cx;
end
if(ax > cx)
    b = ax;
else
    b = cx;
end

w = x;
v = w;
e = 0.0;
fx = NLLA;
fw = fx;
fv = fw;

for iter = 1:MaxIter
        
    xm = 0.5 * ( a + b );
    
%
%  Check the stopping criterion.
%
    if ( abs ( x - xm ) <= t2 - 0.5 * ( b - a ) )
        ThetaA = ThetaA + x*xi;
        NLLA = fx;
        return
    end
%
%  Fit a parabola.
%
    if(abs(e)>tol1)
        r = ( x - w ) * ( fx - fv );
        q = ( x - v ) * ( fx - fw );
        p = ( x - v ) * q - ( x - w ) * r;
        q = 2.0 * ( q - r );

        if ( q > 0.0 )
            p = - p;
        end

        q = abs ( q );

        etemp = e;
        e = d;

        if ( abs ( p ) >= abs ( 0.5 * q * etemp ) || p <= q * ( a - x ) || p >= q * ( b - x ) )
            if ( x > xm)
                e = a-x;
            else
                e = b-x;
            end
            d = GOLD*e;
        else
      %
      %  Take the parabolic interpolation step.
      %
      
            d = p / q;
            u = x + d;
%
%  F must not be evaluated too close to A or B.
%
            if ( ( u - a ) < t2 || ( b - u ) < t2 )

                if ( x < xm )
                    d = tol1;
                else
                    d = - tol1;
                end

            end
        end
%  A golden-section step.
%
    else

        if ( x > xm )
            e = a - x;
        else
            e = b - x;
        end

        d = GOLD * e;

    end
%
%  F must not be evaluated too close to X.
%
    if ( abs ( d )>= tol1 )
        u = x + d;
    elseif ( d >0.0 )
        u = x + tol1;
    else
        u = x - tol1;
    end
    Thetau = ThetaA +u*xi;
    fu = AnalyticalNLL(Thetau,ForecastsSP,Gamma0,Gamma,Returns,SigmaSA,SSP,Xinf,Yields);
    
%
%  Update A, B, V, W, and X.
%
    if ( fu <= fx )

        if ( u >= x )
            a = x;
        else
            b = x;
        end

        v = w;
        fv = fw;
        w = x;
        fw = fx;
        x = u;
        fx = fu;
        
    else

        if ( u < x )
            a = u;
        else
            b = u;
        end

        if ( fu <= fw || w == x )
            v = w;
            fv = fw;
            w = u;
            fw = fu;
        elseif ( fu <= fv || v == x || v== w )
            v = u;
            fv = fu;
        end

    end

end

error('Too many iterations in Brent');
end
function [a,b,c,NLLAa,NLLAb,NLLAc] = AnalyticalBracket(ForecastsSP,Gamma0,Gamma,Returns,SigmaSA,SSP,ThetaA,xi,Xinf,Yields)
%Brackets initial minimum for Brent search
%   
    L = 0.05;% percentage upper/lower initial guess
    a = -L;
    b = 0;
    c = L;
    ThetaAa = ThetaA + a*xi;
    ThetaAc = ThetaA + c*xi;
    NLLAa = AnalyticalNLL(ThetaAa,ForecastsSP,Gamma0,Gamma,Returns,SigmaSA,SSP,Xinf,Yields);
    NLLAb = AnalyticalNLL(ThetaA,ForecastsSP,Gamma0,Gamma,Returns,SigmaSA,SSP,Xinf,Yields);
    NLLAc = AnalyticalNLL(ThetaAc,ForecastsSP,Gamma0,Gamma,Returns,SigmaSA,SSP,Xinf,Yields);
    
    %if a and c already bracket b then return
    if( NLLAa > NLLAb && NLLAc > NLLAb)
        return
    end
    %if not widen Bracket.
    if(NLLAa<=NLLAb)
        while(NLLAa <= NLLAb)
            c = b;
            b = a;
            NLLAc = NLLAb;
            NLLAb = NLLAa;
            a = 2*a;
            ThetaAa = ThetaA + a*xi;
            NLLAa = AnalyticalNLL(ThetaAa,ForecastsSP,Gamma0,Gamma,Returns,SigmaSA,SSP,Xinf,Yields);
        end
    else
        while(NLLAc <= NLLAb)
            a = b;
            b = c;
            NLLAa = NLLAb;
            NLLAb = NLLAc;
            c = 2*c;
            ThetaAc = ThetaA + c*xi;
            NLLAc = AnalyticalNLL(ThetaAc,ForecastsSP,Gamma0,Gamma,Returns,SigmaSA,SSP,Xinf,Yields);
        end
    end
end