function[LogLikelihood,SigmaRnp,SigmaRzp,SigmaRetnp,SigmaRetzp,VolE,ExpectedReturns,XReturns,ReturnVol,YieldVol,LL] = CalculateLogLikelihood(Delta,Forecasts,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Prices,PricesH,Returns,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xinf,Yields)
    % Finds SigmaR and LogLikelihood  
    [Nobs,Nbonds] = size(Yields);
    Nobsnp = 277;
    Nobszp = Nobs - Nobsnp;
    Rmin = 0.0001;
    MPnp = [Theta(25:27)
             Theta(28:30)
             Theta(31:33)];
    MPzp = [Theta(34:36)
             Theta(37:39)
             Theta(40:42)];
    ThPnp = -logm(MPnp)/H;
    ThPzp = -logm(MPzp)/H;
    SigmaPnp = SigmaOU(H,ThPnp',Snp);
    SigmaPzp = SigmaOU(H,ThPzp',Szp);
    ModelYields = zeros(Nobs+1,Nbonds);
    YieldsD = zeros(6,Nbonds);
    D = zeros(6,3);
    DY1 = zeros(3,Nbonds);
    DY2 = zeros(3,Nbonds);
    ExpectedReturns = zeros(Nobs+1,Nbonds);
    XReturns = zeros(Nobs+1,Nbonds);
    ReturnVol = zeros(Nobs+1,Nbonds);
    YieldVol = zeros(Nobs+1,Nbonds);
    D(1,:) = [Delta(1),0,0];
    D(2,:) = -D(1,:);
    D(3,:) = [0,Delta(2),0];
    D(4,:) = -D(3,:);
    D(5,:) = [0,0,Delta(3)];
    D(6,:) = -D(5,:);
    IS = [ImplicitStates
          Xinf];%Add Xinf to the end of the implicitstates for equilibrium calculations
    for t = 1:Nobs+1;
        ModelYields(t,:)= Interpolate(Delta,IS(t,:),MaxD,MinD,Nbonds,Prices);
        ModelYieldsH = InterpolateH(Delta,H,IS(t,:),MaxD,MinD,Nbonds,PricesH);
        Riskfree = max(Gamma0+IS(t,:)*Gamma,Rmin);
        if( t <= Nobsnp)
            DState = (Xinf-IS(t,:))*(eye(3)-MPnp);
            SigmaXi = SigmaPnp;
        else
            DState = (Xinf-IS(t,:))*(eye(3)-MPzp);
            SigmaXi = SigmaPzp;   
        end
              
        %calculate the shifted yield curves
        for k = 1:6;
            YieldsD(k,:) = Interpolate(Delta,IS(t,:)+D(k,:),MaxD,MinD,Nbonds ,Prices);
        end
        
        for j = 1:3;
            DY1(j,:) = (YieldsD(2*j-1,:) - YieldsD(2*j,:))/(2*Delta(j));
            DY2(j,:) = (YieldsD(2*j-1,:) + YieldsD(2*j,:)-2*ModelYields(t,:))/(Delta(j)^2);
        end
        EY = ModelYieldsH + DState*DY1 + 0.5*diag(SigmaXi)'*DY2;
        for n = 1:Nbonds
            ExpectedReturns(t,n) = (n*ModelYields(t,n)-(n-H)*EY(n))/H;
            ReturnVol(t,n) = (n-H)*sqrt(diag(DY1(:,n)'*SigmaXi*DY1(:,n))/H)';
            YieldVol(t,n) = sqrt(diag(DY1(:,n)'*SigmaXi*DY1(:,n))/H)';
        end
        XReturns(t,:) = ExpectedReturns(t,:)-Riskfree*ones(1,Nbonds);
    end
    
    %LogLikelihood calculation
    ER = ExpectedReturns(1:Nobs,[2 10 30]);
    Ret = Returns(:,[2 10 30]);
    Residuals = Yields(:,2:end) - ModelYields(1:Nobs,2:end);
    Residualsnp = Residuals(1:Nobsnp,:);
    Residualszp = Residuals(Nobsnp+1:Nobs,:);
    VolE = std(Residuals,1);
    SigmaRnp = sqrt(trace(Residualsnp*Residualsnp')/((Nbonds-1)*Nobsnp));
    SigmaRzp = sqrt(trace(Residualszp*Residualszp')/((Nbonds-1)*Nobszp));
    ResidualsS = Forecasts-ImplicitStates;
    [~,Nret] = size(Ret);
    ResidualsRetnp = Ret(1:Nobsnp,:) - ER(1:Nobsnp,:);
    ResidualsRetzp = Ret(Nobsnp+1:Nobs,:) - ER(Nobsnp+1:Nobs,:);
    ResidualsRet = [ResidualsRetnp
                    ResidualsRetzp];
    SigmaRetnp = ResidualsRetnp'*ResidualsRetnp/Nobsnp;
    SigmaRetzp = ResidualsRetzp'*ResidualsRetzp/Nobszp;
    LL = zeros(Nobs,1);
    LL(1:Nobsnp,1) = -0.5*(3*log(2*pi)+log(det(SigmaSnp)))-0.5*(Nret*log(2*pi)+log(det(SigmaRetnp)))-0.5*((Nbonds-1)*(log(2*pi)+log(SigmaRnp^2)));
    LL(Nobsnp+1:Nobs,1) = -0.5*(3*log(2*pi)+log(det(SigmaSzp)))-0.5*(Nret*log(2*pi)+log(det(SigmaRetzp)))-0.5*((Nbonds-1)*(log(2*pi)+log(SigmaRzp^2)));
    for t = 1:Nobsnp
        LL(t,1) = LL(t,1)-0.5*ResidualsS(t,:)/SigmaSnp*ResidualsS(t,:)' - 0.5*ResidualsRet(t,:)/SigmaRetnp*ResidualsRet(t,:)'-0.5*Residuals(t,:)*Residuals(t,:)'/SigmaRnp^2;
    end
    for t= Nobsnp+1:Nobs
        LL(t,1) = LL(t,1)-0.5*ResidualsS(t,:)/SigmaSzp*ResidualsS(t,:)' - 0.5*ResidualsRet(t,:)/SigmaRetzp*ResidualsRet(t,:)'-0.5*Residuals(t,:)*Residuals(t,:)'/SigmaRzp^2;
    end
    LogLikelihood = sum(LL);
end