%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% This function finds the ImplicitStates that maximize the LogLikelihood
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
function[NLL,ImplicitStates]=CalculateNLL(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xinf,XTol,Yields)
    persistent MaxLogLikelihood Prices PricesH;
    [Nobs,Nbonds] = size(Yields);
    if isempty(MaxLogLikelihood);
        MaxLogLikelihood = 0;
        [Prices,PricesH]=CalculatePrices(Delta,Gamma0,Gamma,H,MaxD,MinD,Snp,Szp,Theta,Nbonds);
    end
    Pswitch = 1;
    CubeMax = MaxD.*Delta;
    CubeMin = MinD.*Delta;
    if Pswitch == 1
        [Prices,PricesH]=CalculatePrices(Delta,Gamma0,Gamma,H,MaxD,MinD,Snp,Szp,Theta,Nbonds);
    end
    [LogLikelihood,SigmaRnp,SigmaRzp,SigmaRetnp,SigmaRetzp] = CalculateLogLikelihood(Delta,Forecasts,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Prices,PricesH,Returns,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xinf,Yields);
    %display(LogLikelihood)
    NLL = -LogLikelihood;
    tLogLikelihood = LogLikelihood;
    tImplicitStates = ImplicitStates;
    
    if LogLikelihood > MaxLogLikelihood;
        %Find optimal ImplicitStates given Theta using fminsearchbnd. Sometimes this search increases the yield curve fitting error, SigmaR,
        %in order to improve the fit to the forecasts.  In any case it increases the Likelihood.
        MPnp = reshape(Theta(25:33),3,3)';
        MPzp = reshape(Theta(34:42),3,3)';
        ThPnp = -logm(MPnp)/H;
        ThPzp = -logm(MPzp)/H;
        SigmaPnp = SigmaOU(H,ThPnp',Snp);
        SigmaPzp = SigmaOU(H,ThPzp',Szp);
        LikelihoodDiff = LogLikelihood-MaxLogLikelihood;
        OldLogLikelihood = LogLikelihood;
        options=optimset('Display','off','TolFun',FTol,'TolX',XTol);%for fminsearchbnd
        while LikelihoodDiff > FTol;
            for iter =1:Nobs;
                ImplicitStates(iter,:)= fminsearchbnd(@CalculateMonthNLL,ImplicitStates(iter,:),CubeMin,CubeMax,options,Delta,Forecasts(iter,:),H,iter,MaxD,MinD,MPnp,MPzp,Prices,PricesH,Returns(iter,:),SigmaRnp,SigmaRzp,SigmaRetnp,SigmaRetzp,SigmaSnp,SigmaSzp,SigmaPnp,SigmaPzp,Xinf,Yields(iter,:));
            end
            [LogLikelihood,SigmaRnp,SigmaRzp,SigmaRetnp,SigmaRetzp] = CalculateLogLikelihood(Delta,Forecasts,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Prices,PricesH,Returns,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xinf,Yields);
            %display(LogLikelihood);
            LikelihoodDiff = LogLikelihood-OldLogLikelihood;
            OldLogLikelihood = LogLikelihood;
        end

        LogLikelihood = CalculateLogLikelihood(Delta,Forecasts,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Prices,PricesH,Returns,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xinf,Yields);
        if(tLogLikelihood > LogLikelihood)
            ImplicitStates = tImplicitStates;
        end
        LogLikelihood = CalculateLogLikelihood(Delta,Forecasts,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Prices,PricesH,Returns,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xinf,Yields);
        display(LogLikelihood)
        save( savefile,'Delta','H','ImplicitStates','LogLikelihood','MaxD','MinD','Prices','PricesH','Theta');
        NLL = -LogLikelihood;
        MaxLogLikelihood = LogLikelihood;
    end
end
function[NLL,ModelYields]=CalculateMonthNLL(ImplicitStatesi,Delta,Forecastsi,H,iter,MaxD,MinD,MPnp,MPzp,Prices,PricesH,Returnsi,SigmaRnp,SigmaRzp,SigmaRetnp,SigmaRetzp,SigmaSnp,SigmaSzp,SigmaPnp,SigmaPzp,Xinf,Yieldsi)
    %For any given month, calculate the joint conditional likelihood of
    %fitting the yield curve and fitting the forecasts and fitting the returns
    [~,Nbonds] = size(Yieldsi);
    Nobsnp = 277;
    YieldsD = zeros(6,Nbonds);
    D = zeros(6,3);
    DY1 = zeros(3,Nbonds);
    DY2 = zeros(3,Nbonds);
    ExpectedReturns = zeros(1,Nbonds);
    D(1,:) = [Delta(1),0,0];
    D(2,:) = -D(1,:);
    D(3,:) = [0,Delta(2),0];
    D(4,:) = -D(3,:);
    D(5,:) = [0,0,Delta(3)];
    D(6,:) = -D(5,:);
    ModelYields = Interpolate(Delta,ImplicitStatesi,MaxD,MinD,Nbonds,Prices);
    ModelYieldsH = InterpolateH(Delta,H,ImplicitStatesi,MaxD,MinD,Nbonds,PricesH);
    Residualsi = Yieldsi(1,2:end)-ModelYields(1,2:end);
    if( iter <= Nobsnp )
        DS = (Xinf-ImplicitStatesi)*(eye(3)-MPnp);
        Sigmaxi = SigmaPnp;
        SigmaS = SigmaSnp;
        SigmaR = SigmaRnp;
        SigmaRet = SigmaRetnp;
    else
        DS = (Xinf-ImplicitStatesi)*(eye(3)-MPzp);
        Sigmaxi = SigmaPzp;   
        SigmaS = SigmaSzp;
        SigmaR = SigmaRzp;
        SigmaRet = SigmaRetzp;
    end
    %calculate the shifted yield curves
    for k = 1:6;
        YieldsD(k,:) = Interpolate(Delta,ImplicitStatesi+D(k,:),MaxD,MinD,+1,Prices);
    end

    for j = 1:3;
        DY1(j,:) = (YieldsD(2*j-1,:) - YieldsD(2*j,:))/(2*Delta(j));
        DY2(j,:) = (YieldsD(2*j-1,:) + YieldsD(2*j,:)-2*ModelYields)/(Delta(j)^2);
    end
    EY = ModelYieldsH + DS*DY1 + 0.5*diag(Sigmaxi)'*DY2;
    for n = 1:Nbonds
        ExpectedReturns(1,n) = (n*ModelYields(n)-(n-H)*EY(n))/H;
    end
   
    PR = ExpectedReturns(:,[2 10 30]);
    Ret = Returnsi(1,[2 10 30]);
    ResidualsReti=Ret-PR;
    NLL = 0.5*((Forecastsi - ImplicitStatesi)/SigmaS*(Forecastsi - ImplicitStatesi)')+0.5*((Nbonds-1)*log(SigmaRnp^2)+Residualsi*Residualsi'/SigmaR^2)+ 0.5*(log(det(SigmaRet))+ResidualsReti/SigmaRet*ResidualsReti');
end