%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% This function creates the lattice-prices
%%%% Using Implicit centered for second derivatives and implicit upwind for
%%%% first derivatives
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function[Prices,PricesH]=CalculatePrices(Delta,Gamma0,Gamma,H,MaxD,MinD,Snp,Szp,Theta,Nbonds)
    %define parmaeters
    if( floor(1/H) ~= 1/H)
        H = 1/floor(1/H);
        display( H );
    end
    Rmin = 0.0001;
    Munp=Theta(1:3);
    ThQnp = [Theta(4:6)
             Theta(7:9)
             Theta(10:12)];
    Muzp=Theta(13:15);
    THQzp = [Theta(16:18)
             Theta(19:21)
             Theta(22:24)];
    
    %calculates shadow rate given the Implicit variables matrix
    CubeD = MaxD-MinD+[1,1,1];
    Dim = prod(CubeD);
    CubeD1=CubeD(1);
    CubeD2=CubeD(2);
    CubeD3=CubeD(3);
    MinD1=MinD(1);
    MinD2=MinD(2);
    MinD3=MinD(3);
    R = zeros(CubeD);
    parfor ii = 1:CubeD1;
        for jj = 1:CubeD2;
            for kk = 1:CubeD3;
                R(ii,jj,kk)=max(Rmin,Gamma0+Delta.*[MinD1+ii-1,MinD2+jj-1,MinD3+kk-1]*Gamma);
            end
        end
    end
    Rmax = max(max(max(R)));
    
    %calculates (mu-x)/delta and (sigma/delta)^2
    M = zeros(CubeD1,CubeD2,CubeD3,3);
    Mup = zeros(CubeD1,CubeD2,CubeD3,3);
    Mdo = zeros(CubeD1,CubeD2,CubeD3,3);
    VD = zeros(CubeD1,CubeD2,CubeD3,3);
    VDnp = 0.5*(diag(Snp)'./(Delta.*Delta));
    VDzp = 0.5*(diag(Szp)'./(Delta.*Delta));
        
    parfor ii = 1:CubeD1;
        for jj = 1:CubeD2;
            for kk = 1:CubeD3;
                if R(ii,jj,kk)== Rmin;
                    M(ii,jj,kk,:) = ((Muzp - Delta.*[MinD1+ii-1,MinD2+jj-1,MinD3+kk-1])*THQzp)./Delta;
                    VD(ii,jj,kk,:) = VDzp;
                else
                    M(ii,jj,kk,:) = ((Munp - Delta.*[MinD1+ii-1,MinD2+jj-1,MinD3+kk-1])*ThQnp)./Delta;
                    VD(ii,jj,kk,:) = VDnp;
                end
             end
        end
    end
    
    parfor ii = 1:CubeD1;
        for jj = 1:CubeD2;
            for kk = 1:CubeD3;
                for n = 1:3;
                    Mup(ii,jj,kk,n) = M(ii,jj,kk,n)/2;
                    Mdo(ii,jj,kk,n) = M(ii,jj,kk,n)/2;
                end
            end
        end
    end
    
    %Boundaries in direction 1
    for jj = 1:CubeD2;
        for kk = 1:CubeD3;
            Mdo(1,jj,kk,1) = 0;
            Mup(1,jj,kk,1) = max(M(1,jj,kk,1),0);
            VD(1,jj,kk,1) = 0;
            Mdo(CubeD1,jj,kk,1) = min(M(CubeD1,jj,kk,1),0);
            Mup(CubeD1,jj,kk,1) = 0;
            VD(CubeD1,jj,kk,1) = 0;
        end
    end
    %Boundaries in direction 2
    for ii = 1:CubeD1;
        for kk = 1:CubeD3;
            Mdo(ii,1,kk,2) = 0;
            Mup(ii,1,kk,2) = max(M(ii,1,kk,2),0);
            VD(ii,1,kk,2) = 0;
            Mdo(ii,CubeD2,kk,2) = min(M(ii,CubeD2,kk,2),0);
            Mup(ii,CubeD2,kk,2) = 0;
            VD(ii,CubeD2,kk,2) = 0;
        end
    end
    %Boundaries in direction 3
    for ii = 1:CubeD1;
        for jj = 1:CubeD2;
            Mdo(ii,jj,1,3) = 0;
            Mup(ii,jj,1,3) = max(M(ii,jj,1,3),0);
            VD(ii,jj,1,3) = 0;
            Mdo(ii,jj,CubeD3,3) = min(M(ii,jj,CubeD3,3),0);
            Mup(ii,jj,CubeD3,3) = 0;
            VD(ii,jj,CubeD3,3) = 0;
        end
    end
    A = zeros(CubeD1,CubeD2,CubeD3,3);
    B = zeros(CubeD1,CubeD2,CubeD3,3);
    C = zeros(CubeD1,CubeD2,CubeD3,3);
    BB = zeros(CubeD1,CubeD2,CubeD3,3);
    D1 = zeros(CubeD1,CubeD2,CubeD3);
    D2 = zeros(CubeD1,CubeD2,CubeD3);
    D3 = zeros(CubeD1,CubeD2,CubeD3);
    
    %dimension 1 parameters
    parfor ii = 1:CubeD1;
        for jj = 1:CubeD2;
            for kk = 1:CubeD3;
                for n =1:3;
                    A(ii,jj,kk,n) = -0.5*H*(VD(ii,jj,kk,n)-Mdo(ii,jj,kk,n));
                    B(ii,jj,kk,n) = 1+0.5*H*(2*VD(ii,jj,kk,n)+Mup(ii,jj,kk,n)-Mdo(ii,jj,kk,n)+R(ii,jj,kk)/3);
                    C(ii,jj,kk,n) = -0.5*H*(VD(ii,jj,kk,n)+Mup(ii,jj,kk,n));
                    BB(ii,jj,kk,n) = 1-B(ii,jj,kk,n);
                end
            end
        end
    end
    
    T=Nbonds/H;
    MeshPrices = zeros([CubeD,T+1]);
    MeshPrices(:,:,:,1)=1;
    TempPrices1 = zeros(CubeD);
    TempPrices2 = zeros(CubeD);
    TempPrices3 = zeros(CubeD);
    
    for t = 1:T;
        %%step 1
       
        for jj = 1:CubeD2;
            for kk = 1:CubeD3;
                D1(:,jj,kk) = TDMult(-A(:,jj,kk,1),BB(:,jj,kk,1),-C(:,jj,kk,1),MeshPrices(:,jj,kk,t));
            end
        end
        for ii = 1:CubeD1
            for kk = 1:CubeD3
                D2(ii,:,kk) = TDMult(-2*A(ii,:,kk,2),2*BB(ii,:,kk,2),-2*C(ii,:,kk,2),MeshPrices(ii,:,kk,t));
            end
        end
        for ii = 1:CubeD1
            for jj = 1:CubeD2
                D3(ii,jj,:) = TDMult(-2*A(ii,jj,:,3), 2*BB(ii,jj,:,3),-2*C(ii,jj,:,3),MeshPrices(ii,jj,:,t));
            end
        end
        D1 = MeshPrices(:,:,:,t) + D1 + D2 + D3;
        for jj = 1:CubeD2;
            for kk = 1:CubeD3;
                TempPrices1(:,jj,kk) = TDMAsolver(A(:,jj,kk,1),B(:,jj,kk,1),C(:,jj,kk,1),D1(:,jj,kk));
            end
        end
        
        %step 2
        D2 = TempPrices1 - 0.5*D2;
        for ii = 1:CubeD1;
            for kk = 1:CubeD3;
                TempPrices2(ii,:,kk) = TDMAsolver(A(ii,:,kk,2),B(ii,:,kk,2),C(ii,:,kk,2),D2(ii,:,kk));
            end
        end
        
        %step 3
        D3 = TempPrices2 - 0.5*D3;
        for ii = 1:CubeD1;
            for jj = 1:CubeD2;
                TempPrices3(ii,jj,:) = TDMAsolver(A(ii,jj,:,3),B(ii,jj,:,3),C(ii,jj,:,3),D3(ii,jj,:));
            end
        end
        
        %Move to next month
        MaxPrice = exp(-H*Rmin*(t+1));
        MinPrice = exp(-H*Rmax*(t+1));
        
        for ii = 1:CubeD1;
            for jj = 1:CubeD2;
                for kk = 1:CubeD3;
                    MeshPrices(ii,jj,kk,t+1)=max(MinPrice,min(MaxPrice,TempPrices3(ii,jj,kk)));
                end
            end
        end    
        %if(t*H == floor(t*H))
        %    display(t*H);
        %end
    end
    PricesH = zeros(Dim,Nbonds);
    for t = 1:Nbonds
        for ii = 1:Dim
            PricesH(ii,t)= MeshPrices(1+floor((ii-1)/(CubeD2*CubeD3)),1+(floor((ii-1)/CubeD3)-CubeD2*floor((ii-1)/(CubeD3*CubeD2))),1+(ii-1-floor((ii-1)/CubeD3)*CubeD3),t/H);
        end
    end
    Prices = zeros(Dim,Nbonds);
    for t = 1:Nbonds;
        for ii = 1:Dim;
            Prices(ii,t) = MeshPrices(1+floor((ii-1)/(CubeD2*CubeD3)),1+(floor((ii-1)/CubeD3)-CubeD2*floor((ii-1)/(CubeD3*CubeD2))),1+(ii-1-floor((ii-1)/CubeD3)*CubeD3),t/H+1);
        end
    end
end
function x = TDMAsolver(a,b,c,d)
%a, b, c are the column vectors for the compressed tridiagonal matrix, d is the right vector
n = length(b); % n is the number of rows
 
% Modify the first-row coefficients
c(1) = c(1) / b(1);    % Division by zero risk.
d(1) = d(1) / b(1);    % Division by zero would imply a singular matrix.
for i = 2:n-1
    temp = b(i) - a(i) * c(i-1);
    c(i) = c(i) / temp;
    d(i) = (d(i) - a(i) * d(i-1))/temp;
end
 
d(n) = (d(n) - a(n) * d(n-1))/( b(n) - a(n) * c(n-1));
 
% Now back substitute.
x(n) = d(n);
for i = n-1:-1:1
    x(i) = d(i) - c(i) * x(i + 1);
end
end
function x = TDMult(a,b,c,d)
%a, b, c are the column vectors for the compressed tridiagonal matrix, d is the right vector
n = length(b); % n is the number of rows
x= zeros(n,1);
x(1,1) = b(1)*d(1)+c(1)*d(2);
for i = 2:n-1
    x(i,1)= a(i)*d(i-1)+b(i)*d(i)+c(i)*d(i+1);
end
x(n,1) = a(n)*d(n-1)+b(n)*d(n);
end