function [CPIF,UEF,NFPF,RMSEF,ForwardYields,ForwardStates,Yields0F] = Forwards( Constantsnp,Constantszp,Delta,Gamma0,Gamma,ImplicitStates,MaxD,MinD,Mnp,Mzp,Prices,States,Yields )
%Forwards Implicit States
    [Nobs,Nbonds]=size(Yields);
    Yields0F = Yields(Nobs,:);
    ForwardYields = zeros(3,Nbonds);
    for k = 1:3;%index of years forward
        for n = 1:Nbonds-k;%index of forward maturity
            ForwardYields(k,n) = ((n+k)*Yields0F(n+k)-k*Yields0F(k))/n;
        end
    end
    
    %search for implicit forward states
    XTolF = eps;
    FTolF = 1e-12;
    options=optimset('Display','off','TolFun',FTolF,'TolX',XTolF);%for fminsearchbnd
    CubeMax = MaxD.*Delta;
    CubeMin = MinD.*Delta;
    ForwardStates = zeros(4,3);
    RMSEF = zeros(3,1);
    for k =1:3;
        exitflag = 0;
        FS = ForwardStates(k,:);%starting guess
        FY = ForwardYields(k,2:Nbonds-k);
        while exitflag < 1;
            [FS,fval,~,exitflag]= lsqnonlin(@CalculateSE,FS,CubeMin,CubeMax,options,Delta,FY,MaxD,MinD,Prices);
        end
        RMSEF(k,1) = sqrt(fval/(Nbonds-1-k));
        ForwardStates(k+1,:) = FS;
    end
    ForwardStates(1,:) = ImplicitStates(Nobs,:);
    %VAR for comparison
    VARStates = zeros(37,3);
    FFmin =0.0025;
    VARStates(1,:) = States(Nobs,:);
    for t = 1:36;
        r = max(Gamma0+VARStates(t,:)*Gamma,0);
        if r < FFmin;
            VARStates(t+1,:) = Constantszp + VARStates(t,:)*Mzp;
        else
            VARStates(t+1,:) = Constantsnp + VARStates(t,:)*Mnp;
        end
    end
    VARSTATES = [VARStates(1,:)
                 VARStates(13,:)
                 VARStates(25,:)
                 VARStates(37,:)];
    
    CPIF = [ForwardStates(:,1),VARSTATES(:,1)];
    UEF = [ForwardStates(:,2),VARSTATES(:,2)];
    NFPF = [ForwardStates(:,3),VARSTATES(:,3)];
end
function[SE]=CalculateSE(ForwardStates,Delta,Forwards,MaxD,MinD,Prices) 
    %Fit the forward curve
    [~,Nbonds] = size(Forwards);
    ModelYields = Interpolate(Delta,ForwardStates,MaxD,MinD,Nbonds+1,Prices);
    ModelYields =ModelYields(1,2:end);%drop the one year
    SE =(Forwards - ModelYields);
end