%Creates graphs for paper
%graphs of Implicit States and Residual Errors
figure(1);
subplot(2,2,1),plot(datenum(Dates),100*Forecasts(:,1),datenum(Dates),100*ImplicitStates(:,1));
datetick('x')
xlabel('Date')
ylabel('Percent')
title('Core CPI Annual Growth Rate')
grid
subplot(2,2,2),plot(datenum(Dates),100*Forecasts(:,2),datenum(Dates),100*ImplicitStates(:,2))
datetick('x')
xlabel('Date')
ylabel('Percent')
title('Unemployment Rate')
grid
subplot(2,2,3),plot(datenum(Dates),100*Forecasts(:,3),datenum(Dates),100*ImplicitStates(:,3))
datetick('x')
legend('Forecast','Implicit','location','Best')
xlabel('Date')
ylabel('Percent')
title('Non-Farm Payrolls Quarterly Growth Rate')
grid
subplot(2,2,4),plot(2:30,10000*VolE);
title('Residual Errors');
xlabel('Maturity')
ylabel('Basis Points')
grid;
[ax1,h1] = suplabel('Non-Linear Model Implicit States','t');
set(h1,'FontWeight','bold','FontSize',14)
saveas(1,'ImplicitStates.eps','psc2');

%Governement minus implict graph
figure(2);
plot(datenum(Dates),100*Dstates)
datetick('x')
legend('CPI','UE','NFP','location','best')
title('Government Minus Implict State Variables','FontWeight','bold','FontSize',14)
xlabel('Date')
ylabel('Percent')
grid
saveas(2,'GSminusIS.eps','psc2');

%%Equilibrium Yield Curve Graph
figure(3)
plot(1:30,100*Equilibrium,1:30,100*EquilibriumA)
title('Equilibrium Yield Curve','FontWeight','bold','FontSize',14)
xlabel('Maturity')
ylabel('Percent')
ylim([0 6]);
legend('Non-linear','Affine','location','Best');
saveas(3,'Equilibrium.eps','psc2');

%Yield Vol
figure(4);
Maturity = [2,10,30];
for n = 1:3
    subplot(1,3,n),plot(datenum(Dates),100*YieldVol(1:Nobs,Maturity(n)),datenum(Dates),100*YieldVolA(Maturity(n)));
    datetick('x')
    title(Maturity(n));
    xlabel('Date')
    xlim([725341 735345]);
    ylabel('Percent Volatility')
    grid
end
legend('Non-Linear','Affine','location','Best')
[ax4,h4] = suplabel('Non-Linear Model Conditional Yield Volatility','t');
set(h4,'FontWeight','bold','FontSize',14)
saveas(4,'Yvol.eps','psc2');

%Conditional expected excess returns
figure(5);
Maturity = [2,10,30];
for n = 1:3
    subplot(1,3,n),plot(datenum(Dates),100*XReturns(1:Nobs,Maturity(n)),datenum(Dates),100*XReturnsA(1:Nobs,Maturity(n)));
    datetick('x')
    title(Maturity(n));
    xlabel('Date')
    xlim([725341 735345]);
    ylabel('Percent')
    grid
end
legend('Non-Linear','Affine','location','Best')
[ax5,h5] = suplabel('Non-Linear and Affine Model Conditional Expected Excess Returns','t');
set(h5,'FontWeight','bold','FontSize',14)
saveas(5,'ExcessReturn.eps','psc2');

%Sharpe ratio
figure(6);
Maturity = [2,10,30];
for n = 1:3
    subplot(1,3,n),plot(datenum(Dates),Sharpe(1:Nobs,Maturity(n)),datenum(Dates),SharpeA(:,Maturity(n)));
    datetick('x')
    title(Maturity(n));
    xlabel('Date')
    xlim([725341 735345]);
    ylabel('Sharpe Ratio')
    grid
end
legend('Non-Linear','Affine','location','Best')
[ax6,h6] = suplabel('Non-Linear Model Conditional Sharpe Ratio','t');
set(h6,'FontWeight','bold','FontSize',14)
saveas(6,'Sharpe.eps','psc2');

%Forward Graph
figure (7);
subplot(2,2,1),bar(0:3,100*CPIF);
title('Implicit Forward and VAR Forecasted CPI')
xlabel('Years Forward')
ylabel('Percent')
subplot(2,2,2),bar(0:3,100*UEF);
title('Implicit Forward and VAR Forecasted UE')
xlabel('Years Forward')
ylabel('Percent')
subplot(2,2,3),bar(0:3,100*NFPF);
legend('Implicit','VAR','location','best')
title('Implicit Forward and VAR Forecasted NFP')
xlabel('Years Forward')
ylabel('Percent')
subplot(2,2,4),bar(10000*RMSEF);
title('Forward Curve Fitting Error');
xlabel('Years Forward')
ylabel('Basis Points')
[ax7,h7] = suplabel('Non-Linear Model Forward Implicit States','t');
set(h7,'FontWeight','bold','FontSize',14)
saveas(7,'Forwards.eps','psc2');

%plot Fed Funds vs. Taylor Rule
FFTR = [ones(Nobs,1),States(1:Nobs,:)]*BetaTR;
Shadow = [ones(Nobs,1),ImplicitStates(1:Nobs,:)]*[Gamma0;Gamma];
figure(8);
plot(datenum(Dates),100*FedFunds,datenum(Dates),100*FFTR,datenum(Dates),100*Shadow);
legend('Fed Funds','Regression','Non-Linear Model','location','best');
grid;
datetick('x');
title('Effective Fed Funds and Two Estimates of the Shadow Rate','fontweight','b','FontSize',14)
xlabel('Date')
ylabel('Percent')
saveas(8,'PolicyRuleGraph.eps','psc2');

%IRS
figure(9);
N = [Nobs,Nobs-9*12,Nobs-18*12,Nobs-27*12];
for t = 1:4
    subplot(2,2,t),plot(2:30,squeeze(IRS(N(t),2:30,1)),2:30,squeeze(IRS(N(t),2:30,2)),2:30,squeeze(IRS(N(t),2:30,3)));
    title(Dates(N(t)));
    xlabel('Maturity')
    ylabel('Basis Points')
    grid
end
legend('CPI','UE','NFP','location','Best')
[ax9,h9] = suplabel('Conditional State IRS','t');
set(h9,'FontWeight','bold','FontSize',14)
saveas(9,'IRS.eps','psc2');

%Duration
figure(10);
N = [Nobs,Nobs-9*12,Nobs-18*12,Nobs-27*12];
for t = 1:4
    subplot(2,2,t),plot(2:30,squeeze(Dur(N(t),2:30,1)),2:30,squeeze(Dur(N(t),2:30,2)),2:30,squeeze(Dur(N(t),2:30,3)));
    title(Dates(N(t)));
    xlabel('Maturity')
    ylabel('Duration')
    grid
end
legend('CPI','UE','NFP','location','Best')
[ax10,h10] = suplabel('Conditional State Duration','t');
set(h10,'FontWeight','bold','FontSize',14)
saveas(10,'Dur.eps','psc2');

%Convexity
figure(11);
N = [Nobs,Nobs-9*12,Nobs-18*12,Nobs-27*12];
for t = 1:4
    subplot(2,2,t),plot(2:30,squeeze(Con(N(t),2:30,1)),2:30,squeeze(Con(N(t),2:30,2)),2:30,squeeze(Con(N(t),2:30,3)));
    title(Dates(N(t)));
    xlabel('Maturity')
    ylabel('Convexity')
    grid
end
legend('CPI','UE','NFP','location','Best')
[ax11,h11] = suplabel('Conditional State Convexity','t');
set(h11,'FontWeight','bold','FontSize',14)
saveas(11,'Convexity.eps','psc2');

%Risk Premiums
figure(12);
N = [Nobs,Nobs-9*12,Nobs-18*12,Nobs-27*12];
for t = 1:4
    subplot(2,2,t),plot(2:30,squeeze(RiskPrem(N(t),2:30,1)),2:30,squeeze(RiskPrem(N(t),2:30,2)),2:30,squeeze(RiskPrem(N(t),2:30,3)));
    title(Dates(N(t)));
    xlabel('Maturity')
    ylabel('Risk Premium')
    grid
end
legend('CPI','UE','NFP','location','Best')
[ax12,h12] = suplabel('Conditional State Risk Premiums','t');
set(h12,'FontWeight','bold','FontSize',14)
saveas(12,'RiskPremium.eps','psc2');

%Unconditional Vol
figure(13);
DYields = Yields(2:Nobs,2:30) - Yields(1:Nobs-1,2:30);
plot(2:30,100*std(DYields)*sqrt(12));
grid
xlabel('Maturity')
ylabel('Percent Volatility')
title('Unconditional Yield Volatility','FontWeight','bold','FontSize',14)
saveas(13,'UYvol.eps','psc2');

%Conditional Yield Vol 2
figure(14);
for t = 1:4
    subplot(2,2,t),plot(2:30,100*YieldVol(N(t),2:30));
    title(Dates(N(t)));
    xlabel('Maturity')
    ylabel('Percent Volatility')
    grid
end
[ax14,h14] = suplabel('Non-Linear Model Conditional Yield Volatility','t');
set(h14,'FontWeight','bold','FontSize',14)
saveas(14,'Yieldvol.eps','psc2');

%TIPS Strategy returns
figure(15)
plot(datenum(DatesTIPS),100*UWCumReturns,datenum(DatesTIPS),100*WCumReturns)
datetick('x')
legend('Unweighted','Signal Weighted','location','best')
title('Comparison of Unweighted and Signal-Weighted TIPS Strategies','FontWeight','bold','FontSize',14)
xlabel('Date')
ylabel('Cumulative Percentage Return')
grid
saveas(15,'TIPSPandL.eps','psc2');

%Principal components
[Nobs,Nbonds]=size(Yields);
figure(16);
plot(2:30,100*PC(:,1),2:30,100*PC(:,2),2:30,100*PC(:,3));
legend('Level','Slope','Curvature','location','Best')
xlabel('Maturity')
ylabel('Basis Points')
title('First Three Principal Components of the Yield Curve','fontweight','b','FontSize',14)
grid
saveas(16,'PCA.eps','psc2');

%Loadings and Normalized government states
figure(17);
PCStates = States(1:Nobs,:);
mu = mean(PCStates);
vol = std(PCStates,1);
NormedStates = (PCStates-ones(size(PCStates))*diag(mu))/diag(vol);
subplot(1,3,1),plot(datenum(Dates),Loadings(:,1),datenum(Dates),NormedStates(:,1))
datetick('x')
legend('Level','CPI','location','Best')
xlabel('Date')
ylabel('Standard Deviations')
title('Normalized CPI and Loading on Level PC')
grid
L2 = -Loadings(:,2);
subplot(1,3,2),plot(datenum(Dates),L2,datenum(Dates),NormedStates(:,2))
datetick('x')
legend('Slope','UE','location','Best')
xlabel('Date')
ylabel('Standard Deviations')
title('Normalized UE and Loading on Slope PC')
grid
subplot(1,3,3),plot(datenum(Dates),Loadings(:,3),datenum(Dates),NormedStates(:,3))
datetick('x')
legend('Curvature','NFP','location','Best')
xlabel('Date')
ylabel('Standard Deviations')
title('Normalized NFP and Loading on Curvature PC')
grid
[ax17,h17] = suplabel('PCA Loadings','t');
set(h17,'FontWeight','bold','FontSize',14)
saveas(17,'LoadingsandNormStates.eps','psc2');

%affine states and rescaled PC loadings
figure(18);
subplot(1,3,1),plot(datenum(Dates),100*ForecastsA(:,1),datenum(Dates),100*ImplicitStatesA(:,1))
datetick('x')
xlabel('Date')
ylabel('Percent')
title('Core CPI Annual Growth Rate')
grid
subplot(1,3,2),plot(datenum(Dates),100*ForecastsA(:,2),datenum(Dates),100*ImplicitStatesA(:,2))
datetick('x')
xlabel('Date')
ylabel('Percent')
title('Unemployment Rate')
grid
subplot(1,3,3),plot(datenum(Dates),100*ForecastsA(:,3),datenum(Dates),100*ImplicitStatesA(:,3))
datetick('x')
legend('Forecast','Implicit','location','Best')
xlabel('Date')
ylabel('Percent')
title('Non-Farm Payrolls Quarterly Growth Rate')
grid
[ax18,h18] = suplabel('Affine Model Implicit States','t');
set(h18,'FontWeight','bold','FontSize',14)
saveas(18,'ImplicitStatesAffine.eps','psc2');

%Affine IRS
figure(19);
plot(1:30,BetaA);
title('Affine Model Conditional State IRS');
xlabel('Maturity')
ylabel('Basis Points')
grid;
legend('CPI','UE','NFP')
saveas(19,'AffineIRS.eps','psc2');