function [IRS,DV01,Dur,Con,RiskPrem] = Instantaneous(Delta,IS,MaxD,MinD,Prices,Theta,Xinf)
%Conditional DV01, Duration and Volatility
    
    Nobs = length(IS);
    [~,Nbonds] = size(Prices);%include one year bond
    MuQnp=Theta(1:3);
    ThQnp = [Theta(4:6)
             Theta(7:9)
             Theta(10:12)];
    MuQzp=Theta(13:15);
    ThQzp = [Theta(16:18)
             Theta(19:21)
             Theta(22:24)];
    ThPnp = [Theta(25:27)
             Theta(28:30)
             Theta(31:33)];
    ThPzp = [Theta(34:36)
             Theta(37:39)
             Theta(40:42)];
    D = zeros(6,3);
    IRS = zeros(Nobs,Nbonds,3);
    DV01 = zeros(Nobs,Nbonds,3);
    Dur = zeros(Nobs,Nbonds,3);
    Con = zeros(Nobs,Nbonds,3);
    XDS = zeros(Nobs,3);
    RiskPrem = zeros(Nobs,Nbonds,3);
    D(1,:) = [Delta(1),0,0];
    D(2,:) = -D(1,:);
    D(3,:) = [0,Delta(2),0];
    D(4,:) = -D(3,:);
    D(5,:) = [0,0,Delta(3)];
    D(6,:) = -D(5,:);
    for t = 1:Nobs;
        if( t <= 277 || t == Nobs)
            Mu = MuQnp;
            ThQ = ThQnp;
            ThP = ThPnp;
        else
            Mu = MuQzp;
            ThQ = ThQzp;
            ThP = ThPzp;
        end
        Yields0 = Interpolate(Delta,IS(t,:),MaxD,MinD,Nbonds,Prices);
        
        %calculate the shifted yield curves
        YieldsD = zeros(6,Nbonds);
        for k = 1:6;
            YieldsD(k,:) = Interpolate(Delta,IS(t,:)+D(k,:),MaxD,MinD,Nbonds,Prices);
        end
        XDS(t,:) = (Xinf-IS(t,:))*ThP - (Mu-IS(t,:))*ThQ;
        for n = 1:Nbonds;
            for j = 1:3;
                IRS(t,n,j) = (YieldsD(2*j-1,n) - YieldsD(2*j,n))/(2*Delta(j));
                DV01(t,n,j) = (exp(-n*YieldsD(2*j-1,n)) - exp(-n*YieldsD(2*j,n)))/(2*Delta(j));
                Dur(t,n,j) = -DV01(t,n,j)/exp(-n*Yields0(1,n));
                Con(t,n,j) = (exp(-n*YieldsD(2*j-1,n)) + exp(-n*YieldsD(2*j,n)) - 2*exp(-n*Yields0(1,n)))/(exp(-n*Yields0(1,n))*Delta(j)^2);
                RiskPrem(t,n,j) = XDS(t,j)*DV01(t,n,j);
            end
        end
    end
end