function[ModelYieldsH,ModelPricesH] = InterpolateH(Delta,H,ImplicitStates,MaxD,MinD,Nbonds,PricesH)
    %interpolate monthly yield curve given Implicit states and the Lattice.
    %Monthly states do not fall on the nodes of the lattice so the the bond
    %prices must be linearally interpolated in each dimension.
    CubeD = MaxD - MinD + [1 1 1];
    Dim = prod(CubeD);
    % low weights for interpolation
    LowWeight = -ImplicitStates./Delta + floor(ImplicitStates./Delta+1);
  
    % weights for interpolation between nodes
    % (ddd),(ddu),(dud),(duu),(udd),(udu),(uud),(uuu),repectively.
   
    Weight(1) =LowWeight(1)*LowWeight(2)*LowWeight(3);
    Weight(2) =LowWeight(1)*LowWeight(2)*(1-LowWeight(3));
    Weight(3) =LowWeight(1)*(1-LowWeight(2))*LowWeight(3);
    Weight(4) =LowWeight(1)*(1-LowWeight(2))*(1-LowWeight(3));
    Weight(5) =(1-LowWeight(1))*LowWeight(2)*LowWeight(3);
    Weight(6) =(1-LowWeight(1))*LowWeight(2)*(1-LowWeight(3));
    Weight(7) =(1-LowWeight(1))*(1-LowWeight(2))*LowWeight(3);
    Weight(8) =(1-LowWeight(1))*(1-LowWeight(2))*(1-LowWeight(3));
    
    % pointers for interpolation point to nodes 
    % (ddd),(ddu),(dud),(duu),(udd),(udu),(uud),(uuu),repectively.
    
    Pointer(1) = CubeD(2)*CubeD(3)*(floor((ImplicitStates(1)/Delta(1))-MinD(1)+1)-1)+CubeD(3)*(floor((ImplicitStates(2)/Delta(2))-MinD(2)+1)-1)+floor((ImplicitStates(3)/Delta(3))-MinD(3)+1);
    Pointer(1) = min(Pointer(1),Dim);
    Pointer(2) = min(Pointer(1)+1,Dim);
    Pointer(3) = min(Pointer(1)+CubeD(3),Dim);
    Pointer(4) = min(Pointer(3)+1,Dim);
    Pointer(5) = min(Pointer(1)+CubeD(2)*CubeD(3),Dim);
    Pointer(6) = min(Pointer(5)+1,Dim);
    Pointer(7) = min(Pointer(5)+CubeD(3),Dim);
    Pointer(8) = min(Pointer(7)+1,Dim);
   
    % interpolate yield based on these weights and pointers.  ModelYield
    % is the bond price before taking logs and dividing by maturity.
    
    ModelPricesH = zeros(1,Nbonds);%prices for a bond maturing every month
    ModelYieldsH = zeros(1,Nbonds);%yields for bonds maturing in months 11,23,35,...,359
    for j=1:Nbonds;
       ModelPricesH(1,j) = 0;
       for k = 1:8;
            ModelPricesH(1,j) = ModelPricesH(1,j)+ Weight(k)*PricesH(Pointer(k),j);%interpolate prices
       end
       ModelYieldsH(1,j)= -log(ModelPricesH(1,j))/(j-H);
    end
end
    