function [NLL,Theta,ImplicitStates ] = Powell( Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xinf,XTol,Yields )
%Powell's direction set method for finding a minimum NLL without derivatives, but with fixed direction set.
%See Numerical Recipes for algorithm.
%Theta is the starting parameter vector on input and the optimal vector on ouput.
%Tol is the convergence tolerance
%MaxIter is the maximum number of iterations
    Ndim = length(Theta);%number of parameters in search
    MaxIter = Ndim+1;%Should converge here
    Iter = 1;
    Xi = eye(Ndim);
    load PowellFed;%Get starting Xi
    if Ldim < Ndim %#ok<NODEF>
        Fdim = Ldim+1;
    else
        Fdim = 25;
    end
    [NLL,ImplicitStates]=CalculateNLL(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xinf,XTol,Yields);
    %display(NLL);
    while Iter <= MaxIter
        Thetat = Theta;
        NLLt = NLL;
        for ii = Fdim:Ndim;%iterate over row vectors
            display(ii)
            [Theta,ImplicitStates,NLL]=Brent(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,NLL,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xi(ii,:),Xinf,XTol,Yields);
            Ldim = ii;
            save PowellFed Iter Xi Ldim
        end
        Fdim = 25;
        if (abs(NLLt-NLL) <= FTol );
            return
        end
        for ii = 1:Ndim-1;
            Xi(ii,:) = Xi(ii+1,:);
        end
        xit = Theta - Thetat;
        Xi(Ndim,:) = xit/norm(xit);%new normalized direction vector
        [Theta,ImplicitStates,NLL]=Brent(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,NLL,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xi(Ndim,:),Xinf,XTol,Yields);
        Iter = Iter +1 ;
        save PowellFed Iter Xi Ldim
    end
end

function [Theta,ImplicitStates,NLL] = Brent(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,NLL,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Theta,xi,Xinf,XTol,Yields)
% Brent seeks a local minimum of a function F(X) in an interval [A,B].
% GOLD is the golden ratio.
%
    GOLD = 0.5 * ( 3.0 - sqrt ( 5.0 ) );
    MaxIter = 100;
    tol1 = XTol;
    t2 = 2.0 * tol1;
    L = 0.02;%percentage range for initial interval
    Ibracket = 1;
    if( Ibracket == 1)
        %Bracket inital minimum
        [a,x,b,NLL,ImplicitStates] = Bracket(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,L,MaxD,MinD,NLL,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Theta,xi,Xinf,XTol,Yields);
    else
        %take the chance that the inital guess brackets the minimum
        a = -L;
        x = 0;
        b = L;
    end

    w = x;
    v = x;
    e = 0.0;
    fx = NLL;
    fw = NLL;
    fv = NLL;

    for iter = 1:MaxIter
        
        xm = 0.5 * ( a + b );
    
    %
    %  Check the stopping criterion.
    %
        if ( abs ( x - xm ) <= t2 - 0.5 * ( b - a ) )
            Theta = Theta + x*xi;
            NLL = fx;
            return
        end
    %
    %  Fit a parabola.
    %
        if(abs(e)>tol1)
            r = ( x - w ) * ( fx - fv );
            q = ( x - v ) * ( fx - fw );
            p = ( x - v ) * q - ( x - w ) * r;
            q = 2.0 * ( q - r );

            if ( q > 0.0 )
                p = - p;
            end

            q = abs ( q );

            etemp = e;
            e = d;

            if ( abs ( p ) >= abs ( 0.5 * q * etemp ) || p <= q * ( a - x ) || p >= q * ( b - x ) )
                if ( x > xm)
                    e = a-x;
                else
                    e = b-x;
                end
                d = GOLD*e;
            else
          %
          %  Take the parabolic interpolation step.
          %

                d = p / q;
                u = x + d;
        %
        %  F must not be evaluated too close to A or B.
        %
                if ( ( u - a ) < t2 || ( b - u ) < t2 )

                    if ( x < xm )
                        d = tol1;
                    else
                        d = - tol1;
                    end

                end
            end
    %  A golden-section step.
    %
        else

            if ( x > xm )
                e = a - x;
            else
                e = b - x;
            end

            d = GOLD * e;

        end
    %
    %  F must not be evaluated too close to X.
    %
        if ( abs ( d )>= tol1 )
            u = x + d;
        elseif ( d >0.0 )
            u = x + tol1;
        else
            u = x - tol1;
        end
        Thetau = Theta +u*xi;
        [fu,ImplicitStatesu] = CalculateNLL(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Thetau,Xinf,XTol,Yields);
    %
    %  Update A, B, V, W, and X.
    %
        if ( fu <= fx )
            if( fx-fu < FTol )%return if improvement is not at least FTol
                ImplicitStates = ImplicitStatesu;
                Theta = Theta + u*xi;
                NLL = fu;
                return
            end
            if ( u >= x )
                a = x;
            else
                b = x;
            end

            v = w;
            fv = fw;
            w = x;
            fw = fx;
            x = u;
            fx = fu;
            ImplicitStates = ImplicitStatesu;
        else

            if ( u < x )
                a = u;
            else
                b = u;
            end

            if ( fu <= fw || w == x )
                v = w;
                fv = fw;
                w = u;
                fw = fu;
            elseif ( fu <= fv || v == x || v== w )
                v = u;
                fv = fu;
            end

        end

    end

    error('Too many iterations in Brent');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [a,b,c,NLL,ImplicitStates] = Bracket(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,L,MaxD,MinD,NLL,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Theta,xi,Xinf,XTol,Yields)
%Brackets initial minimum for Brent search
%   
    a = -L;
    b = 0;
    c = L;
    Thetaa = Theta + a*xi;
    Thetac = Theta + c*xi;
    NLLa = CalculateNLL(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Thetaa,Xinf,XTol,Yields);
    NLLc = CalculateNLL(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Thetac,Xinf,XTol,Yields);
    
    %if a and c already bracket b then return
    if( NLLa > NLL && NLLc > NLL)
        return
    end
    display('widening bracket')
    display(a)
    display(b)
    display(c)
    display(NLLa)
    display(NLL)
    display(NLLc)
    %if not widen Bracket.
    if(NLLa<=NLL)
        while(NLLa <= NLL)
            c = b;
            b = a;
            NLLc = NLL;
            NLL = NLLa;
            a = 2*a;
            Thetaa = Theta + a*xi;
            NLLa = CalculateNLL(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Thetaa,Xinf,XTol,Yields);
            display(a)
            display(b)
            display(c)
            display(NLLa)
            display(NLL)
            display(NLLc)
        end
    else
        while(NLLc <= NLL)
            a = b;
            b = c;
            NLLa = NLL;
            NLL = NLLc;
            c = 2*c;
            Thetac = Theta + c*xi;
            NLLc = CalculateNLL(Delta,Forecasts,FTol,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Returns,savefile,SigmaSnp,SigmaSzp,Snp,Szp,Thetac,Xinf,XTol,Yields);
            display(a)
            display(b)
            display(c)
            display(NLLa)
            display(NLL)
            display(NLLc)
        end
    end
end
