function[PortfolioROR,Signals,UWCumReturns,WCumReturns,Weights] = TIPSPandLCalc(Delta,ImplicitStates,MaxD,MinD,Prices,PricesTIPS,Repo,ScaleFactor,Yields,YieldsTIPS)
%Calculates monthly P&L for buying a real zero, hedging CPI,UE and NFP with a
%portfolio of bracketing nominal zeros and cash.

[Nobs,Nbonds] = size(YieldsTIPS);

%Calculate nominal and real first derivatives
ActualPricesNom = zeros(Nobs,Nbonds);
APplusNom = zeros(Nobs,Nbonds);
ActualPricesTIPS = zeros(Nobs,Nbonds);
APplusTIPS = zeros(Nobs,Nbonds);
states = zeros(6,3);
ModelPricesTIPS = zeros(Nobs,Nbonds+1);
DurationNom = zeros(3,Nobs,Nbonds+1);
DurationTIPS = zeros(3,Nobs,Nbonds+1);
Signals = zeros(Nobs,1);
%Real bond to be purchased
TIPSmat = 10;%10 year bond
for t = 1:Nobs
    [~,ModelPricesTIPS(t,:)] = Interpolate(Delta,ImplicitStates(t,:),MaxD,MinD,Nbonds+1,PricesTIPS);
    for n =1:Nbonds
        ActualPricesNom(t,n) = exp(-(n+1)*Yields(t,n));
        APplusNom(t,n) = exp(-(n+1-1/12)*Yields(t,n));
        ActualPricesTIPS(t,n) = exp(-(n+1)*YieldsTIPS(t,n));
        APplusTIPS(t,n) = exp(-(n+1-1/12)*YieldsTIPS(t,n));
    end
    Signals(t,1) = ModelPricesTIPS(t,TIPSmat)/ActualPricesTIPS(t,TIPSmat-1)-1;
    states(1,:) = [ImplicitStates(t,1)+Delta(1), ImplicitStates(t,2), ImplicitStates(t,3)];
    states(2,:) = [ImplicitStates(t,1)-Delta(1), ImplicitStates(t,2), ImplicitStates(t,3)];
    states(3,:) = [ImplicitStates(t,1), ImplicitStates(t,2)+Delta(2), ImplicitStates(t,3)];
    states(4,:) = [ImplicitStates(t,1), ImplicitStates(t,2)-Delta(2), ImplicitStates(t,3)];
    states(5,:) = [ImplicitStates(t,1), ImplicitStates(t,2), ImplicitStates(t,3)+Delta(3)];
    states(6,:) = [ImplicitStates(t,1), ImplicitStates(t,2), ImplicitStates(t,3)-Delta(3)];
    [~,P0] = Interpolate(Delta,ImplicitStates(t,:),MaxD,MinD,Nbonds+1,Prices);
    P0TIPS = ModelPricesTIPS(t);
    for j = 1:3
        [~,Pup] = Interpolate(Delta,states(2*j-1,:),MaxD,MinD,Nbonds+1,Prices);
        [~,Pdown] =Interpolate(Delta,states(2*j,:),MaxD,MinD,Nbonds+1,Prices);
        [~,PupTIPS] = Interpolate(Delta,states(2*j-1,:),MaxD,MinD,Nbonds+1,PricesTIPS);
        [~,PdownTIPS] = Interpolate(Delta,states(2*j,:),MaxD,MinD,Nbonds+1,PricesTIPS);
        DurationNom(j,t,:) = ( Pup - Pdown )./(2*Delta(j)*P0);
        DurationTIPS(j,t,:) = ( PupTIPS - PdownTIPS )./(P0TIPS*2*Delta(j));
    end
end
%Nominal hedges
HedgesMat = [5,10,15];%hedges are 2, 10 and 30
%Rolling hedged Monthly returns
ActualRORNom = zeros(Nobs,3);
ActualRORTIPS = zeros(Nobs,1);
ND = zeros(3,3);
UWCumReturns = zeros(Nobs,1);
WCumReturns = zeros(Nobs,1);
Weights = zeros(Nobs,3);
RepoWeight = zeros(Nobs,1);
PortfolioROR = zeros(Nobs,1);
for t = 2:Nobs
    for n = 1:3
        ND(:,n) = DurationNom(:,t-1,HedgesMat(n));
    end
    RD = DurationTIPS(:,t-1,TIPSmat);
    W = -ND\RD;%Hedge weights
    Weights(t-1,:) = W';
    RepoWeight(t-1,1) = -1-sum(W);
    ActualRORTIPS(t,1) = APplusTIPS(t,TIPSmat-1)/ActualPricesTIPS(t-1,TIPSmat-1)-1;
    for n = 1:3
        ActualRORNom(t,n) = APplusNom(t,HedgesMat(n)-1)/ActualPricesNom(t-1,HedgesMat(n)-1)-1;
    end
    PortfolioROR(t,1) = ActualRORTIPS(t,1)+RepoWeight(t-1,1)*Repo(t-1)/12;
    for n =1:3
        PortfolioROR(t,1) = PortfolioROR(t,1) + Weights(t-1,n)*ActualRORNom(t,n);
    end
    UWCumReturns(t,1) = UWCumReturns(t-1,1) + PortfolioROR(t,1);
    WCumReturns(t,1) = WCumReturns(t-1,1) + PortfolioROR(t,1)*Signals(t-1,1)/ScaleFactor;
end

end