function[Gamma0,Gamma,STR,DWTR,RhoTR,RsqTR]=TaylorRule(FedFunds,States)
%Taylor Rule Regression
% This code calculates the Taylor Rule coefficients and statistics for the Normal Policy Period    
Nobsnp = 277; %normal policy period observations are from 11/85 through 11/08 = 277 months
XTR = [ones(Nobsnp,1),States(1:Nobsnp,:)];%state data from 10/85 - 10/08; one month data release lag
FFnp = FedFunds(1:Nobsnp);%Fed Funds data from 11/85 - 11/08
BetaTR = XTR'*XTR\XTR'*FFnp;%coefficients
Gamma0= BetaTR(1);
Gamma = BetaTR(2:4);
ResidualsTR = FFnp - XTR*BetaTR;%residuals
[~,DWTR] = dwtest(ResidualsTR,XTR(1:Nobsnp-1,:));
RsqTR = 1-std(ResidualsTR)^2/std(FFnp)^2;

%Calculations of standard errors adjusted for serial correlation using Newey-West
ResidualsLaggedTR = ResidualsTR(1:Nobsnp-1);%lagged residuals
RhoTR = ResidualsLaggedTR'*ResidualsLaggedTR\ResidualsLaggedTR'*ResidualsTR(2:Nobsnp);%serial correlation
ZTR = zeros(Nobsnp,4);
for ii = 1:Nobsnp;
    for k = 1:4
        ZTR(ii,k) = XTR(ii,k)*ResidualsTR(ii,1);
    end
end
XTXTR = XTR'*XTR;
LTR = floor(4*(Nobsnp/100)^0.25);
OmegahatTR = NeweyWest(ZTR,LTR);
VTR = (XTXTR/Nobsnp)\(OmegahatTR/(XTXTR/Nobsnp))/Nobsnp;
STR = sqrt(diag(VTR));
%%End of Taylor Rule Regressions