%This program produces all of the charts and Excel tables data in "A Non-Linear Macroeconomic Term Structure Model"
%Programmed by Scott Richard

%load inputs
clear all

load InputDataFed;
load AnalyticalDataFed;
load ModelDataFed28275;
load ModelDataFed28275SE;

%PCA
[Nobs,Nbonds]=size(Yields);
Y29 = Yields(:,2:Nbonds)-ones(Nobs,1)*mean(Yields(:,2:Nbonds));
[coeff,~,latent] = pca(Y29,'Centered',false);%Matlab pca call
PC = coeff*diag(sqrt(latent));%Rescale PCs
PC = PC(:,1:3);
Loadings = Y29*PC/(PC'*PC);
PCAresids = Y29-Loadings*PC';
VolEPCA = std(PCAresids,1);
%%end PCA

%%Affine Model
[Gamma0,Gamma]=TaylorRule(FedFunds,States);
[NLLA,ImplicitStatesA,AlphaA,BetaA,ExpectedReturnsA,XReturnsA,ReturnVolA]= AnalyticalNLL(ThetaA,ForecastsA,Gamma0,Gamma,Returns,SigmaSA,SA,Xinf,Yields);
LogLikelihoodA = -NLLA;
EquilibriumA = AlphaA+Xinf*BetaA;
%Calculate Fitting Errors
ResidualsA= Yields-ones(Nobs,1)*AlphaA-ImplicitStatesA*BetaA;
VolEA = std(ResidualsA(:,2:end),1);
YieldVolA = sqrt(12*diag(BetaA'*SigmaSA*BetaA))';
%Parse results for Excel

MuQA=ThetaA(1:3);
ThQA=[ThetaA(4:6)
      ThetaA(7:9)
      ThetaA(10:12)];
MPA = [ThetaA(13:15)
        ThetaA(16:18)
        ThetaA(19:21)];
SigmaRA = ThetaA(22);
SRetA = [ThetaA(23:25);
        0,ThetaA(26:27)
        0,0,ThetaA(28)];
ERA = ExpectedReturnsA(:,[2 10 30]);
SharpeA = XReturnsA/diag(ReturnVolA);
SigmaRetA = SRetA'*SRetA;
display(LogLikelihoodA)
display(SigmaRA)
SEMuQA = SEA(1,1:3);
SEThQA = [SEA(1,4:6)
          SEA(1,7:9)
          SEA(1,10:12)];
SMPA = [SEA(13:15)
        SEA(16:18)
        SEA(19:21)];
SigmaRA = ThetaA(22);
SRetA = [SEA(23:25);
        0,SEA(26:27)
        0,0,SEA(28)];

%%Non-linear model coefficients
[LogLikelihood,SigmaRnp,SigmaRzp,SigmaRetnp,SigmaRetzp,VolE,ExpectedReturns,XReturns,ReturnVol,YieldVol,LL] = CalculateLogLikelihood(Delta,Forecasts,Gamma0,Gamma,H,ImplicitStates,MaxD,MinD,Prices,PricesH,Returns,SigmaSnp,SigmaSzp,Snp,Szp,Theta,Xinf,Yields);
display(LogLikelihood)
display(SigmaRnp)
display(SigmaRzp)
Sharpe = XReturns./ReturnVol;
%normal period
MuQnp = Theta(1:3);
ThQnp=[Theta(4:6)
       Theta(7:9)
       Theta(10:12)];
%zero period
MuQzp = Theta(13:15);
ThQzp=[Theta(16:18)
       Theta(19:21)
       Theta(22:24)];
MPnp = reshape(Theta(25:33),3,3)';
MPzp = reshape(Theta(34:42),3,3)';
%Standard Errors
SEMuQnp = SE(1:3);
SEThQnp = [SE(4:6)
           SE(7:9)
           SE(10:12)];
SEMuQzp = SE(13:15);
SEThQzp = [SE(16:18)
           SE(19:21)
           SE(22:24)];
SEMPnp = [SE(25:27)
         SE(28:30)
         SE(31:33)];
SEMPzp = [SE(34:36)
         SE(37:39)
         SE(40:42)];
Equilibrium = Interpolate(Delta,Xinf,MaxD,MinD,Nbonds,Prices);
ER = ExpectedReturns(:,[2 10 30]);
%%End of Non-linear model

%%Actual-Implict State VAR
Dstates = States(1:end-1,:) - ImplicitStates;
XAI = [ones(Nobs-1,1),Dstates(1:Nobs-1,:)];
YAI = Dstates(2:Nobs,:);
[BetaAI,SEAI,MseAI]=lscov(XAI,YAI);
TstatsAI= BetaAI./SEAI;
RmseAI = sqrt(MseAI);
YvarAI = std(YAI).^2;
RsqAI = ones(1,3)-MseAI./YvarAI;
ResidualsAI = YAI - XAI*BetaAI;
[PAI(1),DWAI(1)]=dwtest(ResidualsAI(:,1),XAI,'approximate');
[PAI(2),DWAI(2)]=dwtest(ResidualsAI(:,2),XAI,'approximate');
[PAI(3),DWAI(3)]=dwtest(ResidualsAI(:,3),XAI,'approximate');

%Policy rule
[Gamma0TR,GammaTR,STR,DWTR,RhoTR,RsqTR]=TaylorRule(FedFunds,States);
BetaTR =[Gamma0TR
         GammaTR];
     
%%Conditional DV01, Duration and Volatility
IS = [ImplicitStates
      Xinf];
[IRS,DV01,Dur,Con,RiskPrem] = Instantaneous(Delta,IS,MaxD,MinD,Prices,Theta,Xinf);
%%End of Conditional DV01, Duration and Volatility

%%Forward Implict States
Mnp = expm(-H*Thnp);
Mzp = expm(-H*Thzp);
Constantsnp = Xinf*(eye(3)-Mnp);
Constantszp = Xinf*(eye(3)-Mzp);
[CPIF,UEF,NFPF,RMSEF,ForwardYields,ForwardStates,Yields0F] = Forwards( Constantsnp,Constantszp,Delta,Gamma0,Gamma,ImplicitStates,MaxD,MinD,Mnp,Mzp,Prices,States,Yields );
%%End of Forwards Implicit States

%%TIPS
FirstTIPS = 159;
DatesTIPS = Dates(FirstTIPS:end);
FedFundsTIPS = FedFunds(FirstTIPS:end,:);
ForecastsTIPS = Forecasts(FirstTIPS:end,:);
ImplicitStatesTIPS = ImplicitStates(FirstTIPS:end,:);
RepoTIPS = Repo(FirstTIPS:end,:);
YieldsNom = Yields(FirstTIPS:end,:);
[TIPSNobs,TIPSNbonds]=size(YieldsTIPS);
PricesTIPS = TIPSCalculatePrices(Delta,Gamma0,Gamma,H,MaxD,MinD,Theta,Snp,Szp,TIPSNbonds+1);
[TIPSLogLikelihood,TIPSSigmaR,TIPSVolE,TIPSResiduals,TIPSN,TIPSLL] = TIPSCalculateLogLikelihood(Delta,FedFundsTIPS,ForecastsTIPS,ImplicitStatesTIPS,MaxD,MinD,PricesTIPS,SigmaSnp,SigmaSzp,YieldsTIPS);
TIPSSigmar = 10000*TIPSSigmaR;
%Calculate P&L for two strategies
ScaleFactor = 0.025;
[PortfolioROR,Signals,UWCumReturns,WCumReturns,Weights] = TIPSPandLCalc(Delta,ImplicitStatesTIPS,MaxD,MinD,Prices,PricesTIPS,RepoTIPS,ScaleFactor,YieldsNom,YieldsTIPS);
save TIPSFed28275  TIPSLogLikelihood PortfolioROR PricesTIPS TIPSSigmaR Signals UWCumReturns WCumReturns Weights
%%End TIPS

